Package: HiCaptuRe
Type: Package
Title: HiCaptuRe: Manipulating and integrating Capture Hi-C data
Version: 1.2.0
Authors@R: c(person("Laureano Tomas-Daza", 
                    role = c("aut", "cre"),
                    email = "lauretomas@gmail.com",
                    comment = c(ORCID = "0000-0002-6127-1694")))
URL: https://github.com/LaureTomas/HiCaptuRe
Description: Capture Hi-C is a set of techniques that enable the detection of genomic interactions involving regions of interest, known as baits. By focusing on selected loci, these approaches reduce sequencing costs while maintaining high resolution at the level of restriction fragments. HiCaptuRe provides tools to import, annotate, manipulate, and export Capture Hi-C data. The package accounts for the specific structure of bait–otherEnd interactions, facilitates integration with other omics datasets, and enables comparison across samples and conditions.
BugReports: https://github.com/LaureTomas/HiCaptuRe/issues
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.5.0)
Imports: Biostrings, BSgenome, cli, data.table, dplyr, GenomeInfoDb,
        GenomicInteractions, GenomicRanges, InteractionSet, ggplot2,
        ggpubr, ggVennDiagram, gplots, igraph, IRanges, memoise,
        methods, S4Vectors, stringr, tibble, tidyr, UpSetR, utils
RoxygenNote: 7.3.2
Suggests: BSgenome.Hsapiens.NCBI.GRCh38, knitr, rmarkdown, DT,
        testthat, BiocStyle, kableExtra
biocViews: Epigenetics, HiC, Sequencing, DataImport, Software
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/HiCaptuRe
git_branch: RELEASE_3_23
git_last_commit: 8d22eb4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:36:34 UTC; biocbuild
Author: Laureano Tomas-Daza [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6127-1694>)
Maintainer: Laureano Tomas-Daza <lauretomas@gmail.com>
