# Generated by roxygen2: do not edit by hand

export(annotate_interactions)
export(digest_genome)
export(distance_summary)
export(export_interactions)
export(getByBaits)
export(getByRegions)
export(getParameters)
export(interactionsByBaits)
export(interactionsByRegions)
export(intersect_interactions)
export(load_interactions)
export(peakmatrix2list)
export(plot_distance_summary)
exportClasses(HiCaptuRe)
exportMethods(getByBaits)
exportMethods(getByRegions)
exportMethods(getParameters)
importFrom(BSgenome,getBSgenome)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,matchPattern)
importFrom(Biostrings,replaceAt)
importFrom(GenomeInfoDb,bsgenomeName)
importFrom(GenomeInfoDb,seqlevels)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomeInfoDb,seqnames)
importFrom(GenomicInteractions,GenomicInteractions)
importFrom(GenomicInteractions,anchorOne)
importFrom(GenomicInteractions,anchorTwo)
importFrom(GenomicInteractions,annotateInteractions)
importFrom(GenomicInteractions,annotateRegions)
importFrom(GenomicInteractions,calculateDistances)
importFrom(GenomicInteractions,export.igraph)
importFrom(GenomicInteractions,is.cis)
importFrom(GenomicInteractions,resetAnnotations)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,mcols)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,split)
importFrom(IRanges,IRanges)
importFrom(IRanges,countOverlaps)
importFrom(IRanges,mergeByOverlaps)
importFrom(IRanges,overlapsAny)
importFrom(IRanges,pintersect)
importFrom(IRanges,subsetByOverlaps)
importFrom(InteractionSet,anchorIds)
importFrom(InteractionSet,regions)
importFrom(S4Vectors,elementMetadata)
importFrom(S4Vectors,subjectHits)
importFrom(S4Vectors,width)
importFrom(UpSetR,fromList)
importFrom(UpSetR,upset)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,rbindlist)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_rows)
importFrom(dplyr,coalesce)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,reframe)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,tibble)
importFrom(ggVennDiagram,ggVennDiagram)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggpubr,theme_classic2)
importFrom(gplots,venn)
importFrom(igraph,as_edgelist)
importFrom(igraph,simplify)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(stats,setNames)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(tibble,add_column)
importFrom(tidyr,separate)
importFrom(tidyr,separate_rows)
