# All examples files (ibed1_example.zip, ibed2_example.zip, peakmatrix_example.zip, annotation_example.txt)
# are filtered to only contain information regarding chromosome 19 of the human genome (GRCh38).
# The 2 ibed files contains the interactions of 2 different cell types, while the peakmatrix contains
# the same information but in a single file.
# The annotation_example.txt file contains the annotation for the baits only for chromosome 19.
# The PAR_Homo_sapiens_coordinates.txt contains the genomic coordinates of the pseudo-autosomical regions
# shared between chromosome X and Y for the human genome (GRCh38), so the function digest_genome()
# can properly masked these regions.
