% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-constants.R
\docType{data}
\name{constants}
\alias{constants}
\alias{.IBED_COLS}
\title{Column name constants for format detection and export}
\format{
An object of class \code{character} of length 10.
}
\usage{
.IBED_COLS
}
\description{
`.IBED_COLS` and `.PEAKMATRIX_COLS` store canonical column names for ibed and peakmatrix formats.
These are used in both `.detect_format()` and export functions to avoid repetition and ensure
consistency between import and export.
}
\keyword{internal}
